package de.intevation.gnv.transition;

import de.intevation.artifactdatabase.Config;

import de.intevation.gnv.state.InputData;
import de.intevation.gnv.state.State;

import de.intevation.gnv.state.exception.StateException;

import java.util.Iterator;

import org.apache.log4j.Logger;

import org.w3c.dom.Node;

/**
 * @author Tim Englich <tim.englich@intevation.de>
 *
 */
public class ValueCompareTransition extends TransitionBase {

    /**
     * the logger, used to log exceptions and additonaly information
     */
    private static Logger log = Logger.getLogger(ValueCompareTransition.class);

    private String dataName = null;
    private String dataValue = null;
    private String operator = null;

    /**
     * The UID of this Class.
     */
    private static final long serialVersionUID = -7846722158776823205L;

    /**
     * Constructor
     */
    public ValueCompareTransition() {
        super();
    }

    /**
     * @see de.intevation.gnv.transition.Transition#isValid(de.intevation.gnv.state.State)
     */
    public boolean isValid(State state) {

        try {
            Iterator<InputData> it = state.getInputData().iterator();
            while (it.hasNext()){
                InputData inputData = it.next();
                if (inputData.getName().equals(this.dataName)){
                    boolean returnValue = false;
                    if (operator.equals("equal")){
                        returnValue = this.dataValue.equals(inputData.getValue());
                    }else if (operator.equals("notequal")){
                        returnValue = !this.dataValue.equals(inputData.getValue());
                    }
                    return returnValue;
                }
            }
            if (operator.equals("notequal")){
                // data is not given. So the constraint not Equals is fullfilled.
                return true;
            }
        } catch (StateException e) {
            log.error(e,e);
            return false;
        }
        return false;
    }

    @Override
    public void setup(Node configuration) {
        super.setup(configuration);
        this.dataName = Config.getStringXPath(configuration,"condition/@inputvalue");
        this.dataValue = Config.getStringXPath(configuration,"condition/@value");
        this.operator = Config.getStringXPath(configuration,"condition/@operator");
    }


}
